from admincli.parse import *

CMD_HELP = "help"
CMD_LOGOUT = "logout"

def help_select():
    print("Command: [<ALIAS> =] {} <COLLECTION> [{} <CONDITION>]".format(Query.QUERY_CMD_SELECT, Query.QUERY_CMD_SELECT_WHERE))
    print("")
    print("Examples:")
    print("  - SELECT account")
    print("     ** Selects all information of all accounts")
    print("  - SELECT account(account_uuid, account_type)")
    print("     ** Selects (account_uuid, account_type) of all accounts")
    print("  - SELECT account WHERE account_type=worker")
    print("     ** Selects all information of all accounts that have `account_type` equal to `worker`")
    print("  - SELECT task WHERE parent_job_uuid=<UUID>")
    print("     ** Selects all information of all children tasks of job with `<UUID>`")


def help_create():
    print("create")

def help_delete():
    print("delete")

def help_general():
    print("Supported Commands:")
    print("\t- {}".format(CMD_HELP))
    print("\t- {}".format(CMD_LOGOUT))
    for command in Query.SUPPORTED_QUERY_COMMANDS:
        print("\t- {}".format(command))
    print("")
    print("Type `{} <Command>` for information about the command".format(CMD_HELP))

def help_logout():
    print("logout")

def help(commands):
    if len(commands) == 1:
        help_general()
    else:
        command_to_help = commands[1]
        if command_to_help.lower() == Query.QUERY_CMD_SELECT:
            help_select()
        elif command_to_help.lower() == Query.QUERY_CMD_CREATE:
            help_create()
        elif command_to_help.lower() == Query.QUERY_CMD_DELETE:
            help_delete()
        elif command_to_help.lower() == CMD_LOGOUT:
            help_logout()
        else:
            help_general()



import re
import json
import traceback as tb

from admincli.utils import *
from admincli.api import *

# A global alias table
alias_table = {}
alias_re = re.compile('(^[a-zA-z]+)[a-zA-z0-9]*')

class ParseError(Exception):
    def __init__(self, message):
        super().__init__(message)

class QueryRuntimeError(Exception):
    def __init__(self, message):
        super().__init__(message)

class Collection:
    collection_queryed = []
    collection_eval = []

    def __init__(self, meta, collection):
        global alias_table
        self.meta = meta
        self.friend_name = collection
        # <COLLECTION> must be of the form <NAME>, or <NAME>(<FIELD>,...)
        # If there is left parenthesis in the collection, then there must be a
        # right parenthesis at the end.
        if collection.count("(") > 1 or collection.count(")") > 1:
            raise ParseError("Collection `{}` invalid. Must be of the form <NAME>, or <NAME>(<FIELD>,...)".format(collection))

        collection_name = collection
        fields = []
        lp_index = collection.find("(")
        if lp_index == 0:
            raise ParseError("Collection `{}` invalid. Must be of the form <NAME>, or <NAME>(<FIELD>,...)".format(collection))
        elif lp_index != -1:
            if collection[-1] != ")":
                raise ParseError("Collection `{}` invalid. Must be of the form <NAME>, or <NAME>(<FIELD>,...)".format(collection))
            collection_name = collection[:lp_index]
            fields_raw = collection[lp_index+1:-1].split(",")
            fields = [field.strip() for field in fields_raw if len(field) > 0]

        # Make sure the collection is defined.
        if collection_name not in alias_table and collection_name not in self.meta["tables"]:
            raise ParseError("Collection <NAME> `{}` unrecognized. Must be previously aliased or one of {}".format(collection_name, self.meta["tables"]))

        collection_supported_fields = []
        if collection_name in alias_table:
            aliased_collection = alias_table[collection_name]
            collection_supported_fields = aliased_collection.fields
        else:
            collection_supported_fields = self.meta["table_fields"][collection_name]

        self.collection_supported_fields = collection_supported_fields
        whole_object = False
        # Make sure each fields are supported.
        if len(fields) > 0:
            for field in fields:
                if field not in collection_supported_fields:
                    raise ParseError("Collection <FIELD> `{}` unrecognized. Must be one of {}".format(field, collection_supported_fields))
        else:
            fields = collection_supported_fields
            whole_object = True

        self.collection_name = collection_name
        if self.collection_name in self.meta["tables"]:
            self.real_name = self.collection_name
        else:
            self.real_name = alias_table[self.collection_name].real_name
        self.fields = fields
        self.whole_object = whole_object
        self.primary_key = self.meta["table_primary"][self.real_name]

class Condition:
    def __init__(self, meta, collection, condition, level):
        self.meta = meta
        self.collection = collection

        # <CONDITION> must be of the form:
        # - <FIELD>=<STR>,...
        # - <FIELD>=<QUERY>
        # - <FIELD>=...&<FIELD>=...&...

        parsed_conditions = {}
        sub_conditions = condition.split("&")
        for sub_condition in sub_conditions:
            condition_to_parse = sub_condition
            eq_index = condition_to_parse.find("=")
            if eq_index == -1:
                raise ParseError("Condition `{}` invalid. Must be of the form FIELD>=<STR>,... or <FIELD>=<QUERY> or <FIELD>=...&<FIELD>=...&...".format(
                                    condition_to_parse))

            field = condition_to_parse[:eq_index].strip()
            # Make sure the field is in the collection.
            if field not in self.collection.collection_supported_fields:
                raise ParseError("Condition <FIELD> `{}` unrecognized. Must be in the <COLLECTION> `{}` {}".format(
                                    field,
                                    self.collection.friend_name,
                                    self.collection.collection_supported_fields))

            if field not in parsed_conditions:
                parsed_conditions[field] = []

            current_condition = condition_to_parse[eq_index+1:].strip()
            if current_condition == '':
                raise ParseError("Condition `{}` invalid. Must be of the form FIELD>=<STR>,... or <FIELD>=<QUERY> or <FIELD>=...&<FIELD>=...&...".format(condition_to_parse))
            # Attempt to parse as Query. If failed, treat as plain string.
            try:
                subquery = Query(meta, current_condition, level)
                parsed_conditions[field].append(subquery)
            except:
                values = current_condition.split(",")
                parsed_conditions[field] += values

        self.parsed_conditions = parsed_conditions

    def eval(self, api, cred):
        fields_evaled = {}
        fields = list(self.parsed_conditions.keys())
        for field in fields:
            fields_evaled[field] = []
            values = self.parsed_conditions[field]
            for value in values:
                if isinstance(value, Query):
                    fields_evaled[field] += value.eval(api, cred)
                else:
                    fields_evaled[field].append(value)

        return fields_evaled

class Query:
    QUERY_CMD_SELECT = "select"
    QUERY_CMD_CREATE = "create"
    QUERY_CMD_DELETE = "delete"

    SUPPORTED_QUERY_COMMANDS = [
        QUERY_CMD_SELECT,
        QUERY_CMD_CREATE,
        QUERY_CMD_DELETE
    ]

    QUERY_CMD_SELECT_WHERE = "where"

    RESERVED_KEYWORDS = [
        QUERY_CMD_SELECT,
        QUERY_CMD_CREATE,
        QUERY_CMD_DELETE,
        QUERY_CMD_SELECT_WHERE
    ]

    def __init__(self, meta, query, level):
        global alias_table
        global alias_re
        if query in alias_table:
            self.aliased = True
            self.aliased_collection = alias_table[query]
            self.query_str = query
        else:
            self.aliased = False
            self.query_str = query
            query = query.lower()
            self.meta = meta
            parsed_tokens = {}
            # A query is a space separated string.
            query_tokens = query.split(" ")
            if len(query_tokens) < 2:
                raise ParseError("Query `{}` invalid. Must be of the form [<ALIAS> =] <CMD> <COLLECTION> [WHERE <CONDITION>]".format(query))

            # Determine command and (optional) alias.
            command = query_tokens[0]
            query_left = query_tokens[1:]
            aliased = False
            while True:
                if command not in self.SUPPORTED_QUERY_COMMANDS:
                    # A special case is if the query is aliased. This is only allowed for the
                    # outter-most query statement.
                    if level == 0 and not aliased:
                        if len(query_tokens) < 4:
                            raise ParseError("Query `{}` invalid. Must be of the form [<ALIAS> =] <CMD> <COLLECTION> [WHERE <CONDITION>]".format(query))

                        if query_tokens[1] != "=":
                            raise ParseError("Query `{}` invalid. Must be of the form [<ALIAS> =] <CMD> <COLLECTION> [WHERE <CONDITION>]".format(query))

                        alias = command
                        if not alias_re.match(alias):
                            raise ParseError("Query <ALIAS> `{}` invalid. Must start with an alphabet character and only use alphabet characters and numbers".format(alias))

                        if alias in self.RESERVED_KEYWORDS:
                            raise ParseError("Query <ALIAS> `{}` invalid. `{}` is a reserved keyword".format(alias, alias))

                        if alias in self.meta["tables"]:
                            raise ParseError("Query <ALIAS> `{}` invalid. `{}` is a reserved name".format(alias, alias))

                        command = query_tokens[2]
                        query_left = query_tokens[3:]
                        aliased = True
                        parsed_tokens["alias"] = alias
                        continue
                    elif aliased:
                        raise ParseError("Query `{}` invalid. Must be of the form [<ALIAS> =] <CMD> <COLLECTION> WHERE <CONDITION>]".format(query))
                    else:
                        raise ParseError("Query <CMD> `{}` invalid. <CMD> must be one of {}".format(
                                            command, SUPPORTED_QUERY_COMMANDS))
                else:
                    parsed_tokens["command"] = command
                    break

            # Look for keyword `where`. If `where` is found, then it is the anchor point to
            # locate the <COLLECTION>. Otherwise, the rest of the command is <COLLECTION>.
            index = 0
            collection_str = ' '.join(query_left)
            condition_str = ' '
            for token in query_left:
                if token == self.QUERY_CMD_SELECT_WHERE:
                    collection_str = ' '.join(query_left[:index])
                    condition_str = ' '.join(query_left[index+1:])
                    break
                index += 1

            collection_str = collection_str.strip()
            condition_str = condition_str.strip()
            if collection_str in self.RESERVED_KEYWORDS:
                raise ParseError("Query <COLLECTION> `{}` invalid. `{}` is a reserved keyword".format(collection_str, collection_str))

            # Parse the collection.
            parsed_tokens["collection"] = Collection(self.meta, collection_str)

            # If there is still a condition left, parse the condition.
            if condition_str != '':
                parsed_tokens["condition"] = Condition(self.meta, parsed_tokens["collection"], condition_str, level+1)

            self.tokens = parsed_tokens

    def eval(self, api, cred):
        if self.aliased:
            collection = self.aliased_collection
            return collection.collection_eval
        else:
            # First, evaluation the condition if exists.
            condition_evaled = {}
            if "condition" in self.tokens:
                condition_evaled = self.tokens["condition"].eval(api, cred)

            query_data = {
                "command" : self.tokens["command"],
                "condition" : json.dumps(condition_evaled)
            }

            query_results = api.query(cred, self.tokens["collection"].real_name, query_data)
            if query_results is None:
                raise QueryRuntimeError("Query Execution Failure at `{}`".format(self.query_str))

            collection = self.tokens["collection"]
            collection.collection_queryed = query_results
            if "alias" in self.tokens:
                alias_table[self.tokens["alias"]] = collection

            # If the collection specifies the whole object, replace that with the primary key.
            if collection.whole_object:
                values = []
                for result in query_results:
                    values.append(result[collection.primary_key])
            else:
                values = []
                header = collection.fields
                for value in query_results:
                    for item in header:
                        values.append(value[item])

            collection.collection_eval = values
            return values

    def get_collection(self):
        if self.aliased:
            return self.aliased_collection
        else:
            return self.tokens["collection"]

def query(api, cred, command):
    global alias_table
    try:
        q = Query(cred["meta"], command, 0)
        q.eval(api, cred)
        collection = q.get_collection()

        # Print the collection.
        header = collection.fields
        table = [header]
        for value in collection.collection_queryed:
            row = []
            for item in header:
                row.append(value[item])
            table.append(row)

        print_table(table)
    except Exception as e:
        print_error("Query Error", str(e))
        return



import json
import requests

from admincli.utils import *

class MasterServerApi:
    def __init__(self, master_endpoint):
        self.master_endpoint = master_endpoint

    def login(self, access_id, password):
        resp = requests.post("{}/api/v1/admin/login/".format(self.master_endpoint),
                             data={'access_id':access_id, 'password':password})

        resp_data = json.loads(resp.text)
        if resp.ok == False:
            print_error(resp_data.get("error"), resp_data.get("message"))
            return None
        else:
            auth_token = TokenAuth(resp_data.get('auth_token'))
            account_data = resp_data.get('account_data')
            meta = resp_data.get('meta')
            return {"auth_token" : auth_token, "account_data" : account_data, "meta" : meta}

    def logout(self, cred):
        requests.post("{}/api/v1/admin/logout/".format(self.master_endpoint), auth=cred["auth_token"])
        return

    def query(self, cred, collection, query_data):
        resp = requests.post("{}/api/v1/admin/query/{}/".format(self.master_endpoint, collection), data=query_data, auth=cred["auth_token"])
        resp_data = json.loads(resp.text)
        if resp.ok == False:
            print_error(resp_data.get("error"), resp_data.get("message"))
            return None
        else:
            return resp_data


import os
import signal
import sys
import argparse

from admincli.api import *
from admincli.help import *
from admincli.parse import *

cred = None
api = None

def clear_screen():
    os.system('cls' if os.name == 'nt' else 'clear')
    return

def message_box(messages):
    print("*" * 80)
    print("**" + " " * 76 + "**")
    for line in messages:
        length = len(line)
        space_left = (76 - length) // 2
        space_right = 76 - length - space_left
        print("**{}{}{}**".format(" " * space_left, line, " " * space_right))

    print("**" + " " * 76 + "**")
    print("*" * 80)
    return

def goodbye(sig, frame):
    clear_screen()
    global cred
    global api
    message_box(['Thank you for using Distributed Deep Learning Admin Portal'])
    if cred is not None:
        api.logout(cred)
    sys.exit(0)
    return

def welcome():
    clear_screen()
    welcome_lines = [
        "Welcome to Distributed Deep Learning Admin Portal",
        "Type '{}' for help".format(CMD_HELP),
        "Press 'Ctrl-C' to exit"
    ]
    message_box(welcome_lines)
    return

def get_prompt(cred):
    prompt = ">>> "
    if cred["account_data"]["is_super_admin"]:
        user = "(Super Admin)"
    else:
        user = "(Admin)"
    return "{} {}".format(user, prompt)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--master_endpoint_override',
        type=str,
        help='override to master server url')

    parser.add_argument(
        '--access_id',
        type=str,
        help='access id to log in to the admin portal')

    parser.add_argument(
        '--password',
        type=str,
        help='password to log in to the admin portal')

    parser.add_argument(
        '--command',
        type=str,
        help='command to execute')

    FLAGS, _ = parser.parse_known_args()
    if FLAGS.master_endpoint_override is None:
        master_endpoint = "http://dtf-masterserver-dev.us-west-1.elasticbeanstalk.com"
    else:
        master_endpoint = FLAGS.master_endpoint_override

    admin_id = None
    if FLAGS.access_id is not None:
        admin_id = FLAGS.access_id

    admin_password = None
    if FLAGS.password is not None:
        admin_password = FLAGS.password

    command = None
    if FLAGS.command is not None:
        command = FLAGS.command

    print(FLAGS)

    # Register for crtl-c
    signal.signal(signal.SIGINT, goodbye)
    if command is None:
        # Print Welcome message...
        welcome()
    global cred
    global api
    api = MasterServerApi(master_endpoint)
    print("Connect to: {}".format(master_endpoint))
    while True:
        if cred is None:
            if admin_id is None:
                admin_id = input("Access Id: ")
            if admin_password is None:
                admin_password = input("Password: ")
            cred = api.login(admin_id, admin_password)
            if cred is None:
                continue
        else:
            # Wait for console input...
            if command is None:
                console_input = input(get_prompt(cred))
            else:
                console_input = command

            console_input_splits = console_input.split(";")
            for console_input_split in console_input_splits:
                print(console_input_splits)
                commands = console_input_split.split()
                if len(commands) == 0:
                    continue
                elif commands[0].lower() == CMD_HELP:
                    help(commands)
                elif commands[0].lower() == CMD_LOGOUT:
                    api.logout(cred)
                    clear_screen()
                    message = ["You Have Been Logged Out"]
                    message_box(message)
                    admin_id = None
                    admin_password = None
                    cred = None
                else:
                    query(api, cred, console_input_split)
            if command is not None:
                if api is not None and cred is not None:
                    api.logout(cred)
                return

main()


import requests

class TokenAuth(requests.auth.AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers['Authorization'] = 'Token ' + self.token
        return r

def print_error(error, message):
    print("[Error] {}".format(error))
    print("[Error Details] {}".format(message))
    return

def print_invalid_command(command, message):
    print("Invalid Syntax for Command `{}`: {}. Type `{} {}` for more information.".format(
            command,
            message,
            MasterServerApi.CMD_HELP,
            command))
    return

def print_table(table, vertical_split="||", horizontal_split="-"):
    s = [[str(e) for e in row] for row in table]
    lens = [max(map(len, col)) for col in zip(*s)]
    fmt = ('  '+vertical_split+' ').join('{{:{}}}'.format(x) for x in lens)
    output = [fmt.format(*row) for row in s]
    print(horizontal_split*len(output[0]))
    print(output[0])
    print(horizontal_split*len(output[0]))
    print('\n'.join(output[1:]))
    return

def print_accounts_as_table(account_data):
    header = ["UUID", "Type", "Created At", "Email", "Capability"]
    table = [header]
    for account_datum in account_data:
        account_out = []
        account_out.append(account_datum["account_uuid"])
        account_out.append(account_datum["account_type"])
        account_out.append(account_datum["created_at"])
        if account_datum["account_type"] == "worker":
            account_out.append("N/A")
            worker_cap = []
            if account_datum["machine_spec_cpu"] != 0:
                worker_cap.append("cpu")
            if account_datum["machine_spec_gpu"] != 0:
                worker_cap.append("gpu")
            account_out(",".join(worker_cap))
        elif account_datum["account_type"] == "user":
            account_out.append(account_datum["email"])
            account_out.append("N/A")
        else:
            account_out.append("N/A")
            account_out.append("N/A")
        table += [account_out]
    print_table(table, '|', '=')



import setuptools

version_keyword = "Version"
version = "0.0.5"
with open("README.md", "r") as fh:
    long_description = fh.read()

    print("Version: {}".format(version))
    setuptools.setup(
        name="ddladmin",
        version=version,
        author="Shen Wang",
        author_email = 'nedlitex0053@gmail.com',
        description="cli for distributed deep learning admin portal",
        long_description=long_description,
        long_description_content_type="text/markdown",
        packages=setuptools.find_packages(),
        entry_points={
            'console_scripts': [
                'ddladmin=admincli.cli:main',
            ],
        },
        install_requires=[
            'requests'
        ],
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
    )